# 2017-Sep-10. Created by Maxim T. 
$scriptPath = Split-Path -parent $MyInvocation.MyCommand.Definition; 
. ($scriptPath + "\psFunctions.ps1");     # Adding script with reusable functions
. ($scriptPath + "\psSetVariables.ps1");  # Adding script to assign values to common variables

$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); 
(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Google Historical".PadRight(28);
$quoteIDFile = $dataRootFolder + "\QuotesIntraDay\GoogleIntraday.txt"; if (Test-Path $quoteIDFile) { Remove-Item $quoteIDFile;} # Removing intraday file before each load

$listStart = $config.IndexOf("<Google>"); $listEnd = $config.IndexOf("</Google>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {"<Google> Symbol list is empty. Exiting script." | Out-File $logFile -Encoding OEM -Append; exit(1);}
$symbolList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on

$urlBase = "http://finance.google.com/finance/historical?q=@@Symbol@@&startdate=@@StartDate@@&output=csv"; # http://www.google.com/finance/historical?q=NYSEARCA%3APWC&startdate=Jan+11%2C+2014&output=csv
$mths = @("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
ForEach($sLine in $symbolList) { # For each symbol
    $ret = GetSymbolInfo $sLine $quotesFolder $minDate; $symbol = $ret[0]; $nextDate = $ret[1]; $symbolMaxDate = $ret[2]; $symbolMaxDateAdj = $ret[3]; $symbolQuoteFile = $ret[4]; $lastQuote = $ret[5];
    "Symbol: $symbol. Next date: $nextDate. Quote file: $symbolQuoteFile" | Out-File $logFile -Encoding OEM -Append;
    if ($nextDate -gt $symbolMaxDate) { "  We already have data up to maximum configuration date of $symbolMaxDate. Will not request new data for this symbol." | Out-File $logFile -Encoding OEM -Append; continue; } 
    $year = $nextDate.Substring(0,4); $month = [int]$nextDate.Substring(5,2) - 1; $day = $nextDate.Substring(8,2); $mthNotShifted = [int] $month+1; # Spliting date into parts. Month is 0 based

     # ========== Get quotes from website - Start ==============
     $startDate=$mths[$month]+"+"+[int] $day+"%2C+"+$year; # Jan+1%2C+2015
     $url = $urlBase.Replace("@@Symbol@@",$symbol).Replace("@@StartDate@@",$startDate);
    "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append; $webpage = ""; $reqRows=0;
     $wc = new-object system.net.WebClient; $reqCount++;
     try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
     catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; continue; } # if attempt to get webpage failed go to next symbol
     
     $quotesTxt = [System.Text.Encoding]::ASCII.GetString($webpage); $quotesTxt = $quotesTxt.Replace("???",""); # This variable now contains downloaded quotes
     if ($quotesTxt.length -le 45) { $reqFailed++; "  $symbol - Not Found (return empty file) `r`n" | Out-File $logFile -Encoding OEM -Append; continue; } # Check if data received makes sense. If request size is less than 45 bytes (header length is 42), then there is something wrong with data received. Ignoring it, going to next symbol
     if ($quotesTxt.Contains("<html>")) {$reqFailed++; "  $symbol - Not Found (return html) `r`n" | Out-File $logFile -Encoding OEM -Append;  continue;} # Result is html file, ignore such file
     $reqSucceed++; # If reached this point, that means symbol request was successful, go next to copy received data into local file
     # ========== Get quotes from website - End. Quote is in $quotesTxt ==============

     $ql = $quotesTxt.Split("`n"); #Splitting text file into lines
     ForEach($q in $ql) { # for each quote line
        if ($q.trim() -ne "" -and !$q.StartsWith("Date")) { #ignore quote lines without data
            $qParts = $q.Split(","); $qDate=$qParts[0]; $qPrice = $qParts[4].REPLACE("-","0");
            if ($qDate.Length -eq 8) {$qDate="0"+$qDate}; # quote date adding leading zero if required 5/06/2015 -> 05/06/2015
            $mthName = $qDate.Substring(3,3); $qMonth= "0"+($mths.IndexOf($mthName) +1); $qMonth = $qMonth.Substring($qMonth.length-2,2); # converting month name into number: Feb->02
            if($qDate.Substring(7,2) -le "50") {$qYear="20"+$qDate.Substring(7,2)} else {$qYear="19"+$qDate.Substring(7,2)} # converting 2char year into 4 char year YY->YYYY (adding century)
            $qDateFormatted = $qYear + "-" + $qMonth +"-" + $qDate.Substring(0,2);
            if($qDateFormatted -le $symbolMaxDateAdj -and $qDateFormatted -ge $nextDate) { # If quote date is before or equal to maximum symbol date
                $qDateFormatted + "," + $qPrice + "," + $symbol | Out-File $symbolQuoteFile -Encoding OEM -Append;
                if ($qDateFormatted -gt $lastQuote) {$lastQuote = $qDateFormatted ;} $reqRows++;
            }
        }
     }

    if ($lastQuote -lt $nextDate) {"Done: $symbol. No new quotes found. `r`n" | Out-File $logFile -Encoding OEM -Append;} # No new records where loaded
    else {"Done: $symbol (from: $nextDate). Record count: $reqRows. Last quote: $lastQuote `r`n" | Out-File $logFile -Encoding OEM -Append; $reqRowsT+=$reqRows;}
}
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed/Rows: $reqCount/$reqSucceed/$reqFailed/$reqRowsT. Duration: $duration`r`n" | Out-File $logFile -Encoding OEM -append;
$logSummary + ". Quotes Requested/Succeed/Failed/Rows: $reqCount/$reqSucceed/$reqFailed/$reqRowsT. Duration: $duration";
