# 2017-Sep-10. Created by Maxim T. 
$scriptPath = Split-Path -parent $MyInvocation.MyCommand.Definition; 
. ($scriptPath + "\psFunctions.ps1");     # Adding script with reusable functions
. ($scriptPath + "\psSetVariables.ps1");  # Adding script to assign values to common variables

$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); 
(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Google Intraday".PadRight(28);
$quoteIDFile = $dataRootFolder + "\QuotesIntraDay\GoogleIntraday.txt"; if (Test-Path $quoteIDFile) { Remove-Item $quoteIDFile;} # Removing intraday file before each load

$sl1IsEmpty = $false; $sl2IsEmpty = $false; $symbolList = @();
$listStart = $config.IndexOf("<Google>"); $listEnd = $config.IndexOf("</Google>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {"<Google> Symbol list is empty. " | Out-File $logFile -Encoding OEM -Append; $sl1IsEmpty=$true;}
if (!$sl1IsEmpty) {$symbolList += @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));} #list of symbols we will work on

$listStart = $config.IndexOf("<GoogleWeb>"); $listEnd = $config.IndexOf("</GoogleWeb>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {"<GoogleWeb> Symbol list is empty. " | Out-File $logFile -Encoding OEM -Append; $sl2IsEmpty=$true;}
if (!$sl2IsEmpty) {$symbolList += @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));} #list of symbols we will work on

if ($sl1IsEmpty -and $sl2IsEmpty) {"Go & GoW empty. Exiting script. " | Out-File $logFile -Encoding OEM -Append; exit(1);}
$symbolList = @($symbolList | ?{$a=$_.split(","); ($a[3] -eq $null -or $a[3] -eq "" -or $a[3] -eq "Y") -and ($a[2] -eq "" -or $a[2] -eq $null)}); # by default if value is empty or y, then get intra-day quotes. Also MaxDate for symbol should not be specified
"Symbol count: " + $symbolList.count + ". MinDate: $minDate" | Out-File $logFile -Encoding OEM -Append;

$urlBase = "https://finance.google.com/finance?q=@@Symbol@@";   $mths = @("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");

ForEach($sLine in $symbolList) { # For each symbol
    $ret = GetSymbolInfo $sLine $quotesFolder $minDate; $symbol = $ret[0]; $nextDate = $ret[1]; $symbolMaxDate = $ret[2]; $symbolMaxDateAdj = $ret[3]; $symbolQuoteFile = $ret[4]; $lastQuote = $ret[5];
    "Symbol: " + $symbol.PadLeft(12) + ". Next date: $nextDate. Quote file: $symbolQuoteFile" | Out-File $logFile -Encoding OEM -Append; 
    if ($nextDate -gt $symbolMaxDate) { "  We already have data up to maximum configuration date of $symbolMaxDate. Will not request new data for this symbol." | Out-File $logFile -Encoding OEM -Append; continue; } 

     # ========== Get latest quote from website - Start ==============
     $url = $urlBase.Replace("@@Symbol@@",$symbol); 
    "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append; $webpage = ""; $reqRows=0; 
     $wc = new-object system.net.WebClient; $reqCount++;
     try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
     catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; continue; } # if attempt to get webpage failed go to next symbol
     
     $quotesTxt = [System.Text.Encoding]::ASCII.GetString($webpage); $quotesTxt = $quotesTxt.Replace("???",""); # This variable now contains downloaded quotes

    # ---------------------------------------------
    # Identifying price
    # ---------------------------------------------
    #Price is in this string: <span class="pr"><span id="ref_696698_l">15.24</span>
    $startStr = "<span class=`"pr`">"; $endStr = "</span>";
    
    if ($quotesTxt.Contains($startStr)) {$quotesTxt = $quotesTxt.Substring($quotesTxt.IndexOf($startStr) + $startStr.Length); if ($quotesTxt.Length -gt 600) {$quotesTxt = $quotesTxt.Substring(0, 600);} } # This now includes price and date
    else {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '<span class='pr'>' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; continue; } # if that symbol has no quotes, then record errors

    if (!$quotesTxt.Contains($endStr)) {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '</span>' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; continue; }# if that symbol has no quotes, then record errors
    $priceTxt = $quotesTxt.Substring(0,$quotesTxt.IndexOf($endStr));

    $startStr = "<span id=";
    if (!$priceTxt.Contains($startStr)) {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '<span id=' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; continue; }# if that symbol has no quotes, then record errors
    $priceTxt = $priceTxt.Substring($priceTxt.IndexOf($startStr) + $startStr.Length);

     $startStr = ">";
     if (!$priceTxt.Contains($startStr)) {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '>' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; continue;} # if that symbol has no quotes, then record errors
     $priceTxt = $priceTxt.Substring($priceTxt.IndexOf($startStr) + $startStr.Length).Replace(",","");
     "  Found price: $priceTxt" | Out-File $logFile -Encoding OEM -Append;
     $reqSucceed++;

    # ---------------------------------------------
    # Identifying date
    # ---------------------------------------------
    # Date: <span class=nwp>Nov 21 - Close</span>  # or date: <div id="ref_22144_elt">Nov 21, 7:59PM EST&nbsp;&nbsp;<div class=mdata-dis>  # or date: <span class=nwp>Real-time:&nbsp;<span id="ref_1982937_ltt">1:53PM EST
    $dateTxt = $quotesTxt.Replace("`r","").Replace("`n","");  $dateFound = $true;
    $startStr = "<span class=nwp>"; $dateVal = (Get-Date).ToString("yyyy-MM-dd");
    if ($dateTxt.Contains($startStr)) { 
        if ($dateTxt.Substring($dateTxt.IndexOf($startStr) + $startStr.Length).Replace("`r","").Replace("`n","").Length -gt 7) {
            $dateVal = $dateTxt.Substring($dateTxt.IndexOf($startStr) + $startStr.Length,7).Replace("`r","").Replace("`n",""); # This now includes date
            "  Date found in <span class=nwp>: '$dateVal'" | Out-File $logFile -Encoding OEM -Append;
        }
        else {$dateFound = $false;  "  Date found not found, using todays date." | Out-File $logFile -Encoding OEM -Append;}
    }
    else {
         $startStr = "_elt`">";
         if ($dateTxt.Contains($startStr)) {
            if ($dateTxt.Substring($dateTxt.IndexOf($startStr) + $startStr.Length).Replace("`r","").Replace("`n","").Length -gt 7) {
                $dateVal = $dateTxt.Substring($dateTxt.IndexOf($startStr) + $startStr.Length,7).Replace("`r","").Replace("`n",""); # This now includes date
                "  Date found after string '_elt': '$dateVal'" | Out-File $logFile -Encoding OEM -Append;
            }
            else {$dateFound = $false;  "  Date found not found, using todays date." | Out-File $logFile -Encoding OEM -Append;}
         }
         else {$dateFound = $false;  "  Date found not found, using todays date." | Out-File $logFile -Encoding OEM -Append;}
    }
    
    $date = (Get-Date).ToString("yyyy-MM-dd");
    $mthName = $dateVal.Substring(0,3); # Date is like: Nov 20,... - so first 3 characters contain month name
    if ($dateFound -and $mths.IndexOf($mthName) -ne -1) {
        $year = (Get-Date).ToString("yyyy");
        $month = ($mths.IndexOf($mthName) + 1).ToString(); if ($mths.IndexOf($mthName) + 1 -lt 10) {$month = "0" + $month;}
        $day = $dateVal.Substring(4,2).Replace(" ","").Replace(",","");
        if ($day -match "^[\d]+$") {if ($day.Length -eq 1) {$day = "0" + $day;} $date = $year + "-" + $month + "-" + $day;} 
    }

    $dayOfWeek = ([datetime]::ParseExact($date,"yyyy-MM-dd",$null)).DayofWeek.ToString().Substring(0,3)

    if($date -ge $nextDate -and $dayOfWeek -ne "Sun" -and $dayOfWeek -ne "Sat") { # If quote date is after or equal to date we already have
        $date + "," + $priceTxt + "," + $symbol | Out-File $quoteIDFile -Encoding OEM -Append; $reqRowsT++;
    }
}

$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed/Rows: $reqCount/$reqSucceed/$reqFailed/$reqRowsT. Duration: $duration`r`n" | Out-File $logFile -Encoding OEM -append;
$logSummary + ". Quotes Requested/Succeed/Failed/Rows: $reqCount/$reqSucceed/$reqFailed/$reqRowsT. Duration: $duration";

